@@ -2,6 +2,7 @@
 
 /* 
  * Copyright (C) 2001 Havoc Pennington
+ * Copyright (C) 2002 Red Hat, Inc.
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,11 +23,23 @@
 #ifndef META_DISPLAY_H
 #define META_DISPLAY_H
 
+#ifndef PACKAGE
+#error "config.h not included"
+#endif
+
 #include <glib.h>
 #include <X11/Xlib.h>
 #include "eventqueue.h"
 #include "common.h"
 
+#ifdef HAVE_STARTUP_NOTIFICATION
+#include <libsn/sn.h>
+#endif
+
+#ifdef HAVE_XSYNC
+#include <X11/extensions/sync.h>
+#endif
+
 #define meta_XFree(p) do { if ((p)) XFree ((p)); } while (0)
 
 /* this doesn't really belong here, oh well. */
@@ -49,6 +62,9 @@ typedef struct _MetaUISlave    MetaUISlave;
 typedef struct _MetaWindow     MetaWindow;
 typedef struct _MetaWorkspace  MetaWorkspace;
 
+typedef struct _MetaWindowPropHooks MetaWindowPropHooks;
+typedef struct _MetaGroupPropHooks  MetaGroupPropHooks;
+
 typedef void (* MetaWindowPingFunc) (MetaDisplay *display,
 				     Window       xwindow,
 				     gpointer     user_data);
@@ -104,10 +120,6 @@ struct _MetaDisplay
   Atom atom_net_client_list_stacking;
   Atom atom_net_wm_state_skip_taskbar;
   Atom atom_net_wm_state_skip_pager;
-  Atom atom_win_workspace;
-  Atom atom_win_layer;
-  Atom atom_win_protocols;
-  Atom atom_win_supporting_wm_check;
   Atom atom_net_wm_icon_name;
   Atom atom_net_wm_icon;
   Atom atom_net_wm_icon_geometry;
@@ -118,12 +130,11 @@ struct _MetaDisplay
   Atom atom_net_active_window;
   Atom atom_metacity_restart_message;
   Atom atom_net_wm_strut;
-  Atom atom_win_hints;
   Atom atom_metacity_reload_theme_message;
   Atom atom_metacity_set_keybindings_message;
   Atom atom_net_wm_state_hidden;
   Atom atom_net_wm_window_type_utility;
-  Atom atom_net_wm_window_type_splashscreen;
+  Atom atom_net_wm_window_type_splash;
   Atom atom_net_wm_ping;
   Atom atom_net_wm_pid;
   Atom atom_wm_client_machine;
@@ -149,6 +160,15 @@ struct _MetaDisplay
   Atom atom_net_wm_action_close;
   Atom atom_net_wm_state_above;
   Atom atom_net_wm_state_below;
+  Atom atom_net_startup_id;
+  Atom atom_metacity_toggle_verbose;
+  Atom atom_metacity_update_counter;
+  Atom atom_sync_counter;
+  Atom atom_gnome_panel_action;
+  Atom atom_gnome_panel_action_main_menu;
+  Atom atom_gnome_panel_action_run_dialog;
+  Atom atom_metacity_sentinel;
+  Atom atom_net_wm_strut_partial;
   
   /* This is the actual window from focus events,
    * not the one we last set
@@ -162,14 +182,12 @@ struct _MetaDisplay
   /* Most recently focused list. Always contains all
    * live windows.
    */
-  GList *mru_list;
-  
-  GList *workspaces;
+  GList *mru_list;  
 
-  guint showing_desktop : 1;
   guint static_gravity_works : 1;
   
   /*< private-ish >*/
+  guint error_trap_synced_at_last_pop : 1;
   MetaEventQueue *events;
   GSList *screens;
   GHashTable *window_ids;
@@ -184,7 +202,6 @@ struct _MetaDisplay
   Window no_focus_window;
   
   /* for double click */
-  int double_click_time;
   Time last_button_time;
   Window last_button_xwindow;
   int last_button_num;
@@ -204,6 +221,9 @@ struct _MetaDisplay
 
   /* Pending autoraise */
   guint       autoraise_timeout_id;
+
+  /* Alt+click button grabs */
+  unsigned int window_grab_modifiers;
   
   /* current window operation */
   MetaGrabOp  grab_op;
@@ -213,24 +233,36 @@ struct _MetaDisplay
   int         grab_button;
   int         grab_initial_root_x;
   int         grab_initial_root_y;
-  int         grab_current_root_x;
-  int         grab_current_root_y;
+  int         grab_latest_motion_x;
+  int         grab_latest_motion_y;
   gulong      grab_mask;
   guint       grab_have_pointer : 1;
   guint       grab_have_keyboard : 1;
   MetaRectangle grab_initial_window_pos;
-  MetaRectangle grab_current_window_pos;
   MetaResizePopup *grab_resize_popup;
   GTimeVal    grab_last_moveresize_time;
-  
+  Time        grab_motion_notify_time;
+
+  /* we use property updates as sentinels for certain window focus events
+   * to avoid some race conditions on EnterNotify events
+   */
+  int         sentinel_counter;
+
+#ifdef HAVE_XKB
+  int         xkb_base_event_type;
+#endif
+#ifdef HAVE_XSYNC
+  /* alarm monitoring client's _METACITY_UPDATE_COUNTER */
+  XSyncAlarm  grab_update_alarm;
+#endif
 
   /* Keybindings stuff */
   MetaKeyBinding *screen_bindings;
   int             n_screen_bindings;
   MetaKeyBinding *window_bindings;
   int             n_window_bindings;
-  int min_keycode;
-  int max_keycode;
+  unsigned int min_keycode;
+  unsigned int max_keycode;
   KeySym *keymap;
   int keysyms_per_keycode;
   XModifierKeymap *modmap;
@@ -249,6 +281,34 @@ struct _MetaDisplay
 
   /* Managed by group.c */
   GHashTable *groups_by_leader;
+
+  /* currently-active window menu if any */
+  MetaWindowMenu *window_menu;
+  MetaWindow *window_with_menu;
+
+  /* Managed by window-props.c */
+  MetaWindowPropHooks *prop_hooks;
+
+  /* Managed by group-props.c */
+  MetaGroupPropHooks *group_prop_hooks;
+  
+#ifdef HAVE_STARTUP_NOTIFICATION
+  SnDisplay *sn_display;
+#endif
+#ifdef HAVE_XSYNC
+  int xsync_event_base;
+  int xsync_error_base;
+#define META_DISPLAY_HAS_XSYNC(display) ((display)->xsync_event_base != 0)
+#else
+#define META_DISPLAY_HAS_XSYNC(display) FALSE
+#endif
+#ifdef HAVE_SHAPE
+  int shape_event_base;
+  int shape_error_base;
+#define META_DISPLAY_HAS_SHAPE(display) ((display)->shape_event_base != 0)
+#else
+#define META_DISPLAY_HAS_SHAPE(display) FALSE
+#endif
 };
 
 gboolean      meta_display_open                (const char  *name);
@@ -285,12 +345,6 @@ GSList*     meta_display_list_windows        (MetaDisplay *display);
 MetaDisplay* meta_display_for_x_display  (Display     *xdisplay);
 GSList*      meta_displays_list          (void);
 
-MetaWorkspace* meta_display_get_workspace_by_index        (MetaDisplay   *display,
-                                                           int            index);
-MetaWorkspace* meta_display_get_workspace_by_screen_index (MetaDisplay   *display,
-                                                           MetaScreen    *screen,
-                                                           int            index);
-
 Cursor         meta_display_create_x_cursor (MetaDisplay *display,
                                              MetaCursor   cursor);
 
@@ -329,10 +383,6 @@ void     meta_display_increment_event_serial (MetaDisplay *display);
 
 void     meta_display_update_active_window_hint (MetaDisplay *display);
 
-/* Show/hide the desktop (temporarily hide all windows) */
-void     meta_display_show_desktop   (MetaDisplay *display);
-void     meta_display_unshow_desktop (MetaDisplay *display);
-
 guint32  meta_display_get_current_time (MetaDisplay *display);
 
 /* utility goo */
@@ -359,10 +409,10 @@ typedef enum
 
 } MetaTabList;
 
-GSList* meta_display_get_tab_list (MetaDisplay   *display,
-                                   MetaTabList    type,
-                                   MetaScreen    *screen,
-                                   MetaWorkspace *workspace);
+GList* meta_display_get_tab_list (MetaDisplay   *display,
+                                  MetaTabList    type,
+                                  MetaScreen    *screen,
+                                  MetaWorkspace *workspace);
 
 MetaWindow* meta_display_get_tab_next (MetaDisplay   *display,
                                        MetaTabList    type,
@@ -378,8 +428,21 @@ MetaWindow* meta_display_get_tab_current (MetaDisplay   *display,
 
 int meta_resize_gravity_from_grab_op (MetaGrabOp op);
 
+gboolean meta_grab_op_is_moving   (MetaGrabOp op);
+gboolean meta_grab_op_is_resizing (MetaGrabOp op);
+
 gboolean meta_rectangle_intersect (MetaRectangle *src1,
                                    MetaRectangle *src2,
                                    MetaRectangle *dest);
+gboolean meta_rectangle_equal (const MetaRectangle *src1,
+                               const MetaRectangle *src2);
+
+void meta_display_devirtualize_modifiers (MetaDisplay        *display,
+                                          MetaVirtualModifier modifiers,
+                                          unsigned int       *mask);
+
+void meta_display_increment_focus_sentinel (MetaDisplay *display);
+void meta_display_decrement_focus_sentinel (MetaDisplay *display);
+gboolean meta_display_focus_sentinel_clear (MetaDisplay *display);
 
 #endif
