@@ -84,6 +84,11 @@ static void handle_panel_keybinding   (MetaDisplay    *display,
                                        MetaWindow     *window,
                                        XEvent         *event,
                                        MetaKeyBinding *binding);
+static void handle_toggle_above       (MetaDisplay    *display,
+                                       MetaScreen     *screen,
+                                       MetaWindow     *window,
+                                       XEvent         *event,
+                                       MetaKeyBinding *binding);
 static void handle_toggle_maximize    (MetaDisplay    *display,
                                        MetaScreen     *screen,
                                        MetaWindow     *window,
@@ -134,6 +139,11 @@ static void handle_move_to_workspace  (MetaDisplay    *display,
                                        MetaWindow     *window,
                                        XEvent         *event,
                                        MetaKeyBinding *binding);
+static void handle_move_to_workspace_flip  (MetaDisplay    *display,
+                                            MetaScreen     *screen,
+                                            MetaWindow     *window,
+                                            XEvent         *event,
+                                            MetaKeyBinding *binding);
 static void handle_workspace_switch   (MetaDisplay    *display,
                                        MetaScreen     *screen,
                                        MetaWindow     *window,
@@ -348,6 +358,7 @@ static const MetaKeyHandler screen_handlers[] = {
 static const MetaKeyHandler window_handlers[] = {
   { META_KEYBINDING_WINDOW_MENU, handle_activate_menu, NULL },
   { META_KEYBINDING_TOGGLE_FULLSCREEN, handle_toggle_fullscreen, NULL },
+  { META_KEYBINDING_TOGGLE_ABOVE, handle_toggle_above, NULL },
   { META_KEYBINDING_TOGGLE_MAXIMIZE, handle_toggle_maximize, NULL },
   { META_KEYBINDING_MAXIMIZE, handle_maximize, NULL },
   { META_KEYBINDING_UNMAXIMIZE, handle_unmaximize, NULL },
@@ -381,13 +392,13 @@ static const MetaKeyHandler window_handlers[] = {
     GINT_TO_POINTER (10) },
   { META_KEYBINDING_MOVE_WORKSPACE_12, handle_move_to_workspace,
     GINT_TO_POINTER (11) },
-  { META_KEYBINDING_MOVE_WORKSPACE_LEFT, handle_move_to_workspace,
+  { META_KEYBINDING_MOVE_WORKSPACE_LEFT, handle_move_to_workspace_flip,
     GINT_TO_POINTER (META_MOTION_LEFT) },
-  { META_KEYBINDING_MOVE_WORKSPACE_RIGHT, handle_move_to_workspace,
+  { META_KEYBINDING_MOVE_WORKSPACE_RIGHT, handle_move_to_workspace_flip,
     GINT_TO_POINTER (META_MOTION_RIGHT) },
-  { META_KEYBINDING_MOVE_WORKSPACE_UP, handle_move_to_workspace,
+  { META_KEYBINDING_MOVE_WORKSPACE_UP, handle_move_to_workspace_flip,
     GINT_TO_POINTER (META_MOTION_UP) },
-  { META_KEYBINDING_MOVE_WORKSPACE_DOWN, handle_move_to_workspace,
+  { META_KEYBINDING_MOVE_WORKSPACE_DOWN, handle_move_to_workspace_flip,
     GINT_TO_POINTER (META_MOTION_DOWN) },
   { META_KEYBINDING_RAISE_OR_LOWER, handle_raise_or_lower, NULL},
   { META_KEYBINDING_RAISE, handle_raise, NULL},
@@ -920,7 +931,7 @@ meta_change_keygrab (MetaDisplay *display,
                      unsigned int keycode,
                      int          modmask)
 {
-  int ignored_mask;
+  unsigned int ignored_mask;
 
   /* Grab keycode/modmask, together with
    * all combinations of ignored modifiers.
@@ -937,7 +948,7 @@ meta_change_keygrab (MetaDisplay *display,
   meta_error_trap_push (display);
   
   ignored_mask = 0;
-  while (ignored_mask < (int) display->ignored_modifier_mask)
+  while (ignored_mask <= display->ignored_modifier_mask)
     {
       if (ignored_mask & ~(display->ignored_modifier_mask))
         {
@@ -1694,14 +1705,26 @@ process_keyboard_move_grab (MetaDisplay *display,
   else
     incr = NORMAL_INCREMENT;
 
+  if (keysym == XK_Escape)
+    {
+      /* End move and restore to original position */
+      meta_window_move_resize (display->grab_window,
+                               TRUE,
+                               display->grab_initial_window_pos.x,
+                               display->grab_initial_window_pos.y,
+                               display->grab_initial_window_pos.width,
+                               display->grab_initial_window_pos.height);
+    }
+  
   /* When moving by increments, we still snap to edges if the move
    * to the edge is smaller than the increment. This is because
    * Shift + arrow to snap is sort of a hidden feature. This way
    * people using just arrows shouldn't get too frustrated.
    */
-      
   switch (keysym)
     {
+    case XK_KP_Home:
+    case XK_KP_Prior:
     case XK_Up:
     case XK_KP_Up:
       edge = meta_window_find_next_horizontal_edge (window, FALSE);
@@ -1712,6 +1735,8 @@ process_keyboard_move_grab (MetaDisplay *display,
           
       handled = TRUE;
       break;
+    case XK_KP_End:
+    case XK_KP_Next:
     case XK_Down:
     case XK_KP_Down:
       edge = meta_window_find_next_horizontal_edge (window, TRUE);
@@ -1722,6 +1747,12 @@ process_keyboard_move_grab (MetaDisplay *display,
           
       handled = TRUE;
       break;
+    }
+  
+  switch (keysym)
+    {
+    case XK_KP_Home:
+    case XK_KP_End:
     case XK_Left:
     case XK_KP_Left:
       edge = meta_window_find_next_vertical_edge (window, FALSE);
@@ -1732,6 +1763,8 @@ process_keyboard_move_grab (MetaDisplay *display,
 
       handled = TRUE;
       break;
+    case XK_KP_Prior:
+    case XK_KP_Next:
     case XK_Right:
     case XK_KP_Right:
       edge = meta_window_find_next_vertical_edge (window, TRUE);
@@ -1740,19 +1773,6 @@ process_keyboard_move_grab (MetaDisplay *display,
         x = edge;
       handled = TRUE;
       break;
-
-    case XK_Escape:
-      /* End move and restore to original position */
-      meta_window_move_resize (display->grab_window,
-                               TRUE,
-                               display->grab_initial_window_pos.x,
-                               display->grab_initial_window_pos.y,
-                               display->grab_initial_window_pos.width,
-                               display->grab_initial_window_pos.height);
-      break;
-          
-    default:
-      break;
     }
 
   if (handled)
@@ -2181,10 +2201,6 @@ process_tab_grab (MetaDisplay *display,
       
       if (target_window)
         {
-          meta_topic (META_DEBUG_KEYBINDINGS,
-                      "Ending grab early so we can focus the target window\n");
-          meta_display_end_grab_op (display, event->xkey.time);
-
           meta_topic (META_DEBUG_KEYBINDINGS,
                       "Activating target window\n");
 
@@ -2192,6 +2208,10 @@ process_tab_grab (MetaDisplay *display,
                       target_window->desc);
           meta_window_activate (target_window, event->xkey.time);
 
+          meta_topic (META_DEBUG_KEYBINDINGS,
+                      "Ending grab early so we can focus the target window\n");
+          meta_display_end_grab_op (display, event->xkey.time);
+
           return TRUE; /* we already ended the grab */
         }
       
@@ -2587,7 +2607,10 @@ handle_toggle_desktop (MetaDisplay    *display,
                        MetaKeyBinding *binding)
 {
   if (screen->showing_desktop)
-    meta_screen_unshow_desktop (screen);
+    {
+      meta_screen_unshow_desktop (screen);
+      meta_screen_focus_top_window (screen, NULL);
+    }
   else
     meta_screen_show_desktop (screen);
 }
@@ -2843,6 +2866,22 @@ handle_toggle_fullscreen  (MetaDisplay    *display,
     }
 }
 
+static void
+handle_toggle_above       (MetaDisplay    *display,
+                           MetaScreen     *screen,
+                           MetaWindow     *window,
+                           XEvent         *event,
+                           MetaKeyBinding *binding)
+{
+  if (window)
+    {
+      if (window->wm_state_above)
+        meta_window_unmake_above (window);
+      else
+        meta_window_make_above (window);
+    }
+}
+
 static void
 handle_toggle_maximize    (MetaDisplay    *display,
                            MetaScreen     *screen,
@@ -2974,11 +3013,12 @@ handle_toggle_sticky      (MetaDisplay    *display,
 }
 
 static void
-handle_move_to_workspace  (MetaDisplay    *display,
-                           MetaScreen     *screen,
-                           MetaWindow     *window,
-                           XEvent         *event,
-                           MetaKeyBinding *binding)
+do_handle_move_to_workspace  (MetaDisplay    *display,
+                              MetaScreen     *screen,
+                              MetaWindow     *window,
+                              XEvent         *event,
+                              MetaKeyBinding *binding,
+                              gboolean        flip)
 {
   int which;
   MetaWorkspace *workspace;
@@ -3003,7 +3043,8 @@ handle_move_to_workspace  (MetaDisplay    *display,
     {
       /* Activate second, so the window is never unmapped */
       meta_window_change_workspace (window, workspace);
-      meta_workspace_activate (workspace);
+      if (flip)
+        meta_workspace_activate_with_focus (workspace, window);
     }
   else
     {
@@ -3011,6 +3052,35 @@ handle_move_to_workspace  (MetaDisplay    *display,
     }  
 }
 
+static void
+handle_move_to_workspace  (MetaDisplay    *display,
+                           MetaScreen     *screen,
+                           MetaWindow     *window,
+                           XEvent         *event,
+                           MetaKeyBinding *binding)
+{
+  do_handle_move_to_workspace (display,
+                               screen,
+                               window,
+                               event,
+                               binding,
+                               FALSE);
+}
+static void
+handle_move_to_workspace_flip  (MetaDisplay    *display,
+                                MetaScreen     *screen,
+                                MetaWindow     *window,
+                                XEvent         *event,
+                                MetaKeyBinding *binding)
+{
+  do_handle_move_to_workspace (display,
+                               screen,
+                               window,
+                               event,
+                               binding,
+                               TRUE);
+}
+
 static void 
 handle_raise_or_lower (MetaDisplay    *display,
                        MetaScreen     *screen,
