@@ -148,9 +148,9 @@ const MetaXineramaScreenInfo* meta_screen_get_xinerama_for_rect   (MetaScreen
 const MetaXineramaScreenInfo* meta_screen_get_xinerama_for_window (MetaScreen    *screen,
                                                                    MetaWindow    *window);
 
-gboolean      meta_screen_window_intersects_xinerama (MetaScreen *screen, 
-                                                      MetaWindow *window,
-                                                      int         which_xinerama);
+gboolean      meta_screen_rect_intersects_xinerama (MetaScreen    *screen, 
+                                                    MetaRectangle *window,
+                                                    int            which_xinerama);
 
 const MetaXineramaScreenInfo* meta_screen_get_xinerama_neighbor (MetaScreen *screen,
                                                                  int         which_xinerama,
@@ -188,10 +188,12 @@ void meta_screen_resize (MetaScreen *screen,
                          int         width,
                          int         height);
 
+void     meta_screen_minimize_all_except (MetaScreen *screen,
+                                          MetaWindow *keep);
 
 /* Show/hide the desktop (temporarily hide all windows) */
-void     meta_screen_show_desktop   (MetaScreen *screen);
-void     meta_screen_unshow_desktop (MetaScreen *screen);
+void     meta_screen_show_desktop        (MetaScreen *screen);
+void     meta_screen_unshow_desktop      (MetaScreen *screen);
 
 void     meta_screen_apply_startup_properties (MetaScreen *screen,
                                                MetaWindow *window);
