@@ -2,7 +2,7 @@
 
 /* 
  * Copyright (C) 2001 Havoc Pennington
- * Copyright (C) 2002 Red Hat, Inc.
+ * Copyright (C) 2002, 2003 Red Hat, Inc.
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -183,20 +183,33 @@ window_is_fullscreen_size (MetaWindow *window)
 {
   int i;
 
-  if (window->rect.x <= 0 &&
-      window->rect.y <= 0 &&
-      window->rect.width >= window->screen->width &&
+  if (window->rect.width >= window->screen->width &&
       window->rect.height >= window->screen->height)
-    return TRUE;
+    {
+      /* we use the work area since windows that try to
+       * position at 0,0 will get pushed down by menu panel
+       */
+      MetaRectangle workarea;
+      
+      meta_window_get_work_area_current_xinerama (window, &workarea);
+      if (window->rect.x <= workarea.x &&
+          window->rect.y <= workarea.y)
+        return TRUE;
+    }
   
   i = 0;
   while (i < window->screen->n_xinerama_infos)
     {
-      if (window->rect.x == window->screen->xinerama_infos[i].x_origin &&
-          window->rect.y == window->screen->xinerama_infos[i].y_origin &&
-          window->rect.width >= window->screen->xinerama_infos[i].width &&
+      if (window->rect.width >= window->screen->xinerama_infos[i].width &&
           window->rect.height >= window->screen->xinerama_infos[i].height)
-        return TRUE;
+        {
+          MetaRectangle workarea;
+          
+          meta_window_get_work_area_current_xinerama (window, &workarea);
+          if (window->rect.x <= workarea.x &&
+              window->rect.y <= workarea.y) 
+            return TRUE;
+        }
       
       ++i;
     }
@@ -204,11 +217,25 @@ window_is_fullscreen_size (MetaWindow *window)
   return FALSE;
 }
 
+static gboolean
+is_focused_foreach (MetaWindow *window,
+                    void       *data)
+{
+  if (window->has_focus ||
+      (window == window->display->expected_focus_window))
+    {
+      *((gboolean*) data) = TRUE;
+      return FALSE;
+    }
+  return TRUE;
+}
+
 /* Get layer ignoring any transient or group relationships */
 static MetaStackLayer
 get_standalone_layer (MetaWindow *window)
 {
   MetaStackLayer layer;
+  gboolean focused_transient = FALSE;
   
   switch (window->type)
     {
@@ -229,14 +256,12 @@ get_standalone_layer (MetaWindow *window)
       break;
       
     default:       
+      meta_window_foreach_transient (window,
+                                     is_focused_foreach,
+                                     &focused_transient);
 
-#if 0
-      if (window->has_focus &&
-          meta_prefs_get_focus_mode () == META_FOCUS_MODE_CLICK)
-        layer = META_LAYER_FOCUSED_WINDOW;
-#endif
-      
-      if (window->has_focus &&
+      if ((window->has_focus || focused_transient ||
+           (window == window->display->expected_focus_window)) &&
           (window->fullscreen || window_is_fullscreen_size (window)))
         layer = META_LAYER_FULLSCREEN;
       else if (window->wm_state_above)
@@ -249,37 +274,34 @@ get_standalone_layer (MetaWindow *window)
   return layer;
 }
 
+typedef struct
+{
+  MetaStackLayer max;
+} MaxLayerData;
+
+static gboolean
+max_layer_func (MetaWindow *window,
+                void       *data)
+{
+  MaxLayerData *d = data;
+  MetaStackLayer layer;
+  
+  layer = get_standalone_layer (window);
+  if (layer > d->max)
+    d->max = layer;
+
+  return TRUE;
+}
+
 static MetaStackLayer
 get_maximum_layer_of_ancestor (MetaWindow *window)
 {
-  MetaWindow *w;
-  MetaStackLayer max;
-  MetaStackLayer layer;
-  
-  max = get_standalone_layer (window);
-  
-  w = window;
-  while (w != NULL)
-    {
-      if (w->xtransient_for == None ||
-          w->transient_parent_is_root_window)
-        break;
-      
-      w = meta_display_lookup_x_window (w->display, w->xtransient_for);
-      
-      if (w == window)
-        break; /* Cute, someone thought they'd make a transient_for cycle */
-      
-      /* w may be null... */
-      if (w != NULL)
-        {
-          layer = get_standalone_layer (w);
-          if (layer > max)
-            max = layer;
-        }
-    }
+  MaxLayerData d;
 
-  return max;
+  d.max = get_standalone_layer (window);
+  meta_window_foreach_ancestor (window, max_layer_func, &d);
+  
+  return d.max;
 }
 
 /* Note that this function can never use window->layer only
@@ -488,6 +510,8 @@ add_constraint (Constraint **constraints,
                 MetaWindow  *below)
 {
   Constraint *c;
+
+  g_assert (above->screen == below->screen);
   
   /* check if constraint is a duplicate */
   c = constraints[below->stack_position];
@@ -537,6 +561,8 @@ create_constraints (Constraint **constraints,
 
       if (!WINDOW_IN_STACK (w))
         {
+          meta_topic (META_DEBUG_STACK, "Window %s not in the stack, not constraining it\n",
+                      w->desc);
           tmp = tmp->next;
           continue;
         }
@@ -560,7 +586,8 @@ create_constraints (Constraint **constraints,
             {
               MetaWindow *group_window = tmp2->data;
 
-              if (!WINDOW_IN_STACK (group_window))
+              if (!WINDOW_IN_STACK (group_window) ||
+                  w->screen != group_window->screen)
                 {
                   tmp2 = tmp2->next;
                   continue;
@@ -595,7 +622,8 @@ create_constraints (Constraint **constraints,
           parent =
             meta_display_lookup_x_window (w->display, w->xtransient_for);
 
-          if (parent && WINDOW_IN_STACK (parent))
+          if (parent && WINDOW_IN_STACK (parent) &&
+              parent->screen == w->screen)
             {
               meta_topic (META_DEBUG_STACK, "Constraining %s above %s due to transiency\n",
                           w->desc, parent->desc);
@@ -1297,12 +1325,27 @@ meta_stack_get_below (MetaStack      *stack,
     return NULL;
   else
     return below;
-}                               
+}
 
-MetaWindow*
-meta_stack_get_default_focus_window (MetaStack     *stack,
-                                     MetaWorkspace *workspace,
-                                     MetaWindow    *not_this_one)
+static gboolean
+window_contains_point (MetaWindow *window,
+                       int         root_x,
+                       int         root_y)
+{
+  MetaRectangle rect;
+
+  meta_window_get_outer_rect (window, &rect);
+
+  return POINT_IN_RECT (root_x, root_y, rect);
+}
+
+static MetaWindow*
+get_default_focus_window (MetaStack     *stack,
+                          MetaWorkspace *workspace,
+                          MetaWindow    *not_this_one,
+                          gboolean       must_be_at_point,
+                          int            root_x,
+                          int            root_y)
 {
   /* Find the topmost, focusable, mapped, window.
    * not_this_one is being unfocused or going away, so exclude it.
@@ -1351,12 +1394,16 @@ meta_stack_get_default_focus_window (MetaStack     *stack,
             {
               if (transient_parent == NULL &&
                   not_this_one->xtransient_for != None &&
-                  not_this_one->xtransient_for == window->xwindow)
+                  not_this_one->xtransient_for == window->xwindow &&
+                  (!must_be_at_point ||
+                   window_contains_point (window, root_x, root_y)))
                 transient_parent = window;
 
               if (topmost_in_group == NULL &&
                   not_this_one_group != NULL &&
-                  not_this_one_group == meta_window_get_group (window))
+                  not_this_one_group == meta_window_get_group (window) &&
+                  (!must_be_at_point ||
+                   window_contains_point (window, root_x, root_y)))
                 topmost_in_group = window;
             }
 
@@ -1365,7 +1412,9 @@ meta_stack_get_default_focus_window (MetaStack     *stack,
            * focusing dock, even though docks are stacked higher.
            */
           if (topmost_overall == NULL &&
-              window->type != META_WINDOW_DOCK)
+              window->type != META_WINDOW_DOCK &&
+              (!must_be_at_point ||
+               window_contains_point (window, root_x, root_y)))
             topmost_overall = window;
 
           /* We could try to bail out early here for efficiency in
@@ -1386,6 +1435,26 @@ meta_stack_get_default_focus_window (MetaStack     *stack,
     return topmost_dock;
 }
 
+MetaWindow*
+meta_stack_get_default_focus_window_at_point (MetaStack     *stack,
+                                              MetaWorkspace *workspace,
+                                              MetaWindow    *not_this_one,
+                                              int            root_x,
+                                              int            root_y)
+{
+  return get_default_focus_window (stack, workspace, not_this_one,
+                                   TRUE, root_x, root_y);
+}
+
+MetaWindow*
+meta_stack_get_default_focus_window (MetaStack     *stack,
+                                     MetaWorkspace *workspace,
+                                     MetaWindow    *not_this_one)
+{
+  return get_default_focus_window (stack, workspace, not_this_one,
+                                   FALSE, 0, 0);
+}
+
 GList*
 meta_stack_list_windows (MetaStack     *stack,
                          MetaWorkspace *workspace)
